@@ -17,10 +17,10 @@ import os
 import warnings
 
 import google.api_core.client_options
-from google.auth.credentials import AnonymousCredentials
-from google.cloud._helpers import _LocalStack
-from google.cloud._helpers import _determine_default_project as _base_default_project
-from google.cloud.client import ClientWithProject
+from google.auth.credentials import AnonymousCredentials  # type: ignore
+from google.cloud._helpers import _LocalStack  # type: ignore
+from google.cloud._helpers import _determine_default_project as _base_default_project  # type: ignore
+from google.cloud.client import ClientWithProject  # type: ignore
 from google.cloud.datastore.version import __version__
 from google.cloud.datastore import helpers
 from google.cloud.datastore._http import HTTPDatastoreAPI
@@ -32,13 +32,14 @@ from google.cloud.datastore.transaction import Transaction
 
 try:
     from google.cloud.datastore._gapic import make_datastore_api
-
 except ImportError:  # pragma: NO COVER
-    from google.api_core import client_info
+    from google.api_core import client_info as api_core_client_info
+
+    def make_datastore_api(client):
+        raise RuntimeError("No gRPC available")
 
-    make_datastore_api = None
     _HAVE_GRPC = False
-    _CLIENT_INFO = client_info.ClientInfo(client_library_version=__version__)
+    _CLIENT_INFO = api_core_client_info.ClientInfo(client_library_version=__version__)
 else:
     from google.api_core.gapic_v1 import client_info
 
@@ -737,6 +738,10 @@ class Client(ClientWithProject):
             kwargs["namespace"] = self.namespace
         return Key(*path_args, **kwargs)
 
+    def entity(self, key=None, exclude_from_indexes=()):
+        """Proxy to :class:`google.cloud.datastore.entity.Entity`."""
+        return Entity(key=key, exclude_from_indexes=exclude_from_indexes)
+
     def batch(self):
         """Proxy to :class:`google.cloud.datastore.batch.Batch`."""
         return Batch(self)
