@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,24 +13,24 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 from collections import OrderedDict
 import functools
 import re
 from typing import Dict, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions  # type: ignore
+from google.api_core.client_options import ClientOptions  # type: ignore
+from google.api_core import exceptions as core_exceptions  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import retry as retries  # type: ignore
-from google.auth import credentials  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+OptionalRetry = Union[retries.Retry, object]
+
 from google.cloud.datastore_v1.types import datastore
 from google.cloud.datastore_v1.types import entity
 from google.cloud.datastore_v1.types import query
-
 from .transports.base import DatastoreTransport, DEFAULT_CLIENT_INFO
 from .transports.grpc_asyncio import DatastoreGrpcAsyncIOTransport
 from .client import DatastoreClient
@@ -58,29 +57,55 @@ class DatastoreAsyncClient:
     parse_common_billing_account_path = staticmethod(
         DatastoreClient.parse_common_billing_account_path
     )
-
     common_folder_path = staticmethod(DatastoreClient.common_folder_path)
     parse_common_folder_path = staticmethod(DatastoreClient.parse_common_folder_path)
-
     common_organization_path = staticmethod(DatastoreClient.common_organization_path)
     parse_common_organization_path = staticmethod(
         DatastoreClient.parse_common_organization_path
     )
-
     common_project_path = staticmethod(DatastoreClient.common_project_path)
     parse_common_project_path = staticmethod(DatastoreClient.parse_common_project_path)
-
     common_location_path = staticmethod(DatastoreClient.common_location_path)
     parse_common_location_path = staticmethod(
         DatastoreClient.parse_common_location_path
     )
 
-    from_service_account_file = DatastoreClient.from_service_account_file
+    @classmethod
+    def from_service_account_info(cls, info: dict, *args, **kwargs):
+        """Creates an instance of this client using the provided credentials
+            info.
+
+        Args:
+            info (dict): The service account private key info.
+            args: Additional arguments to pass to the constructor.
+            kwargs: Additional arguments to pass to the constructor.
+
+        Returns:
+            DatastoreAsyncClient: The constructed client.
+        """
+        return DatastoreClient.from_service_account_info.__func__(DatastoreAsyncClient, info, *args, **kwargs)  # type: ignore
+
+    @classmethod
+    def from_service_account_file(cls, filename: str, *args, **kwargs):
+        """Creates an instance of this client using the provided credentials
+            file.
+
+        Args:
+            filename (str): The path to the service account private key json
+                file.
+            args: Additional arguments to pass to the constructor.
+            kwargs: Additional arguments to pass to the constructor.
+
+        Returns:
+            DatastoreAsyncClient: The constructed client.
+        """
+        return DatastoreClient.from_service_account_file.__func__(DatastoreAsyncClient, filename, *args, **kwargs)  # type: ignore
+
     from_service_account_json = from_service_account_file
 
     @property
     def transport(self) -> DatastoreTransport:
-        """Return the transport used by the client instance.
+        """Returns the transport used by the client instance.
 
         Returns:
             DatastoreTransport: The transport used by the client instance.
@@ -94,12 +119,12 @@ class DatastoreAsyncClient:
     def __init__(
         self,
         *,
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         transport: Union[str, DatastoreTransport] = "grpc_asyncio",
         client_options: ClientOptions = None,
         client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
     ) -> None:
-        """Instantiate the datastore client.
+        """Instantiates the datastore client.
 
         Args:
             credentials (Optional[google.auth.credentials.Credentials]): The
@@ -131,7 +156,6 @@ class DatastoreAsyncClient:
             google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                 creation failed for any reason.
         """
-
         self._client = DatastoreClient(
             credentials=credentials,
             transport=transport,
@@ -141,39 +165,40 @@ class DatastoreAsyncClient:
 
     async def lookup(
         self,
-        request: datastore.LookupRequest = None,
+        request: Union[datastore.LookupRequest, dict] = None,
         *,
         project_id: str = None,
         read_options: datastore.ReadOptions = None,
         keys: Sequence[entity.Key] = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.LookupResponse:
         r"""Looks up entities by key.
 
         Args:
-            request (:class:`~.datastore.LookupRequest`):
+            request (Union[google.cloud.datastore_v1.types.LookupRequest, dict]):
                 The request object. The request for
                 [Datastore.Lookup][google.datastore.v1.Datastore.Lookup].
             project_id (:class:`str`):
                 Required. The ID of the project
                 against which to make the request.
+
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            read_options (:class:`~.datastore.ReadOptions`):
+            read_options (:class:`google.cloud.datastore_v1.types.ReadOptions`):
                 The options for this lookup request.
                 This corresponds to the ``read_options`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            keys (:class:`Sequence[~.entity.Key]`):
+            keys (:class:`Sequence[google.cloud.datastore_v1.types.Key]`):
                 Required. Keys of entities to look
                 up.
+
                 This corresponds to the ``keys`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -181,7 +206,7 @@ class DatastoreAsyncClient:
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.LookupResponse:
+            google.cloud.datastore_v1.types.LookupResponse:
                 The response for
                 [Datastore.Lookup][google.datastore.v1.Datastore.Lookup].
 
@@ -200,12 +225,10 @@ class DatastoreAsyncClient:
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
-
         if project_id is not None:
             request.project_id = project_id
         if read_options is not None:
             request.read_options = read_options
-
         if keys:
             request.keys.extend(keys)
 
@@ -218,8 +241,10 @@ class DatastoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded, exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.ServiceUnavailable,
                 ),
+                deadline=60.0,
             ),
             default_timeout=60.0,
             client_info=DEFAULT_CLIENT_INFO,
@@ -233,19 +258,18 @@ class DatastoreAsyncClient:
 
     async def run_query(
         self,
-        request: datastore.RunQueryRequest = None,
+        request: Union[datastore.RunQueryRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.RunQueryResponse:
         r"""Queries for entities.
 
         Args:
-            request (:class:`~.datastore.RunQueryRequest`):
+            request (Union[google.cloud.datastore_v1.types.RunQueryRequest, dict]):
                 The request object. The request for
                 [Datastore.RunQuery][google.datastore.v1.Datastore.RunQuery].
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -253,13 +277,12 @@ class DatastoreAsyncClient:
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.RunQueryResponse:
+            google.cloud.datastore_v1.types.RunQueryResponse:
                 The response for
                 [Datastore.RunQuery][google.datastore.v1.Datastore.RunQuery].
 
         """
         # Create or coerce a protobuf request object.
-
         request = datastore.RunQueryRequest(request)
 
         # Wrap the RPC method; this adds retry and timeout information,
@@ -271,8 +294,10 @@ class DatastoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded, exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.ServiceUnavailable,
                 ),
+                deadline=60.0,
             ),
             default_timeout=60.0,
             client_info=DEFAULT_CLIENT_INFO,
@@ -286,26 +311,26 @@ class DatastoreAsyncClient:
 
     async def begin_transaction(
         self,
-        request: datastore.BeginTransactionRequest = None,
+        request: Union[datastore.BeginTransactionRequest, dict] = None,
         *,
         project_id: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.BeginTransactionResponse:
         r"""Begins a new transaction.
 
         Args:
-            request (:class:`~.datastore.BeginTransactionRequest`):
+            request (Union[google.cloud.datastore_v1.types.BeginTransactionRequest, dict]):
                 The request object. The request for
                 [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
             project_id (:class:`str`):
                 Required. The ID of the project
                 against which to make the request.
+
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -313,7 +338,7 @@ class DatastoreAsyncClient:
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.BeginTransactionResponse:
+            google.cloud.datastore_v1.types.BeginTransactionResponse:
                 The response for
                 [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
 
@@ -332,7 +357,6 @@ class DatastoreAsyncClient:
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
-
         if project_id is not None:
             request.project_id = project_id
 
@@ -352,13 +376,13 @@ class DatastoreAsyncClient:
 
     async def commit(
         self,
-        request: datastore.CommitRequest = None,
+        request: Union[datastore.CommitRequest, dict] = None,
         *,
         project_id: str = None,
         mode: datastore.CommitRequest.Mode = None,
         transaction: bytes = None,
         mutations: Sequence[datastore.Mutation] = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.CommitResponse:
@@ -366,18 +390,20 @@ class DatastoreAsyncClient:
         or modifying some entities.
 
         Args:
-            request (:class:`~.datastore.CommitRequest`):
+            request (Union[google.cloud.datastore_v1.types.CommitRequest, dict]):
                 The request object. The request for
                 [Datastore.Commit][google.datastore.v1.Datastore.Commit].
             project_id (:class:`str`):
                 Required. The ID of the project
                 against which to make the request.
+
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            mode (:class:`~.datastore.CommitRequest.Mode`):
+            mode (:class:`google.cloud.datastore_v1.types.CommitRequest.Mode`):
                 The type of commit to perform. Defaults to
                 ``TRANSACTIONAL``.
+
                 This corresponds to the ``mode`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
@@ -386,10 +412,11 @@ class DatastoreAsyncClient:
                 commit. A transaction identifier is returned by a call
                 to
                 [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
+
                 This corresponds to the ``transaction`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            mutations (:class:`Sequence[~.datastore.Mutation]`):
+            mutations (:class:`Sequence[google.cloud.datastore_v1.types.Mutation]`):
                 The mutations to perform.
 
                 When mode is ``TRANSACTIONAL``, mutations affecting a
@@ -404,10 +431,10 @@ class DatastoreAsyncClient:
 
                 When mode is ``NON_TRANSACTIONAL``, no two mutations may
                 affect a single entity.
+
                 This corresponds to the ``mutations`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -415,7 +442,7 @@ class DatastoreAsyncClient:
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.CommitResponse:
+            google.cloud.datastore_v1.types.CommitResponse:
                 The response for
                 [Datastore.Commit][google.datastore.v1.Datastore.Commit].
 
@@ -434,14 +461,12 @@ class DatastoreAsyncClient:
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
-
         if project_id is not None:
             request.project_id = project_id
         if mode is not None:
             request.mode = mode
         if transaction is not None:
             request.transaction = transaction
-
         if mutations:
             request.mutations.extend(mutations)
 
@@ -461,23 +486,24 @@ class DatastoreAsyncClient:
 
     async def rollback(
         self,
-        request: datastore.RollbackRequest = None,
+        request: Union[datastore.RollbackRequest, dict] = None,
         *,
         project_id: str = None,
         transaction: bytes = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.RollbackResponse:
         r"""Rolls back a transaction.
 
         Args:
-            request (:class:`~.datastore.RollbackRequest`):
+            request (Union[google.cloud.datastore_v1.types.RollbackRequest, dict]):
                 The request object. The request for
                 [Datastore.Rollback][google.datastore.v1.Datastore.Rollback].
             project_id (:class:`str`):
                 Required. The ID of the project
                 against which to make the request.
+
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
@@ -485,10 +511,10 @@ class DatastoreAsyncClient:
                 Required. The transaction identifier, returned by a call
                 to
                 [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
+
                 This corresponds to the ``transaction`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -496,10 +522,9 @@ class DatastoreAsyncClient:
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.RollbackResponse:
-                The response for
-                [Datastore.Rollback][google.datastore.v1.Datastore.Rollback].
-                (an empty message).
+            google.cloud.datastore_v1.types.RollbackResponse:
+                The response for [Datastore.Rollback][google.datastore.v1.Datastore.Rollback].
+                   (an empty message).
 
         """
         # Create or coerce a protobuf request object.
@@ -516,7 +541,6 @@ class DatastoreAsyncClient:
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
-
         if project_id is not None:
             request.project_id = project_id
         if transaction is not None:
@@ -538,11 +562,11 @@ class DatastoreAsyncClient:
 
     async def allocate_ids(
         self,
-        request: datastore.AllocateIdsRequest = None,
+        request: Union[datastore.AllocateIdsRequest, dict] = None,
         *,
         project_id: str = None,
         keys: Sequence[entity.Key] = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.AllocateIdsResponse:
@@ -550,24 +574,25 @@ class DatastoreAsyncClient:
         referencing an entity before it is inserted.
 
         Args:
-            request (:class:`~.datastore.AllocateIdsRequest`):
+            request (Union[google.cloud.datastore_v1.types.AllocateIdsRequest, dict]):
                 The request object. The request for
                 [Datastore.AllocateIds][google.datastore.v1.Datastore.AllocateIds].
             project_id (:class:`str`):
                 Required. The ID of the project
                 against which to make the request.
+
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            keys (:class:`Sequence[~.entity.Key]`):
+            keys (:class:`Sequence[google.cloud.datastore_v1.types.Key]`):
                 Required. A list of keys with
                 incomplete key paths for which to
                 allocate IDs. No key may be
                 reserved/read-only.
+
                 This corresponds to the ``keys`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -575,7 +600,7 @@ class DatastoreAsyncClient:
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.AllocateIdsResponse:
+            google.cloud.datastore_v1.types.AllocateIdsResponse:
                 The response for
                 [Datastore.AllocateIds][google.datastore.v1.Datastore.AllocateIds].
 
@@ -594,10 +619,8 @@ class DatastoreAsyncClient:
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
-
         if project_id is not None:
             request.project_id = project_id
-
         if keys:
             request.keys.extend(keys)
 
@@ -617,11 +640,11 @@ class DatastoreAsyncClient:
 
     async def reserve_ids(
         self,
-        request: datastore.ReserveIdsRequest = None,
+        request: Union[datastore.ReserveIdsRequest, dict] = None,
         *,
         project_id: str = None,
         keys: Sequence[entity.Key] = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.ReserveIdsResponse:
@@ -629,23 +652,24 @@ class DatastoreAsyncClient:
         llocated by Cloud Datastore.
 
         Args:
-            request (:class:`~.datastore.ReserveIdsRequest`):
+            request (Union[google.cloud.datastore_v1.types.ReserveIdsRequest, dict]):
                 The request object. The request for
                 [Datastore.ReserveIds][google.datastore.v1.Datastore.ReserveIds].
             project_id (:class:`str`):
                 Required. The ID of the project
                 against which to make the request.
+
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            keys (:class:`Sequence[~.entity.Key]`):
+            keys (:class:`Sequence[google.cloud.datastore_v1.types.Key]`):
                 Required. A list of keys with
                 complete key paths whose numeric IDs
                 should not be auto-allocated.
+
                 This corresponds to the ``keys`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -653,7 +677,7 @@ class DatastoreAsyncClient:
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.ReserveIdsResponse:
+            google.cloud.datastore_v1.types.ReserveIdsResponse:
                 The response for
                 [Datastore.ReserveIds][google.datastore.v1.Datastore.ReserveIds].
 
@@ -672,10 +696,8 @@ class DatastoreAsyncClient:
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
-
         if project_id is not None:
             request.project_id = project_id
-
         if keys:
             request.keys.extend(keys)
 
@@ -688,8 +710,10 @@ class DatastoreAsyncClient:
                 maximum=60.0,
                 multiplier=1.3,
                 predicate=retries.if_exception_type(
-                    exceptions.DeadlineExceeded, exceptions.ServiceUnavailable,
+                    core_exceptions.DeadlineExceeded,
+                    core_exceptions.ServiceUnavailable,
                 ),
+                deadline=60.0,
             ),
             default_timeout=60.0,
             client_info=DEFAULT_CLIENT_INFO,
@@ -701,6 +725,12 @@ class DatastoreAsyncClient:
         # Done; return the response.
         return response
 
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, exc_type, exc, tb):
+        await self.transport.close()
+
 
 try:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
