@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,28 +13,28 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 from collections import OrderedDict
 from distutils import util
 import os
 import re
-from typing import Callable, Dict, Optional, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
 from google.api_core import client_options as client_options_lib  # type: ignore
-from google.api_core import exceptions  # type: ignore
+from google.api_core import exceptions as core_exceptions  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import retry as retries  # type: ignore
-from google.auth import credentials  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport import mtls  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+OptionalRetry = Union[retries.Retry, object]
+
 from google.cloud.datastore_v1.types import datastore
 from google.cloud.datastore_v1.types import entity
 from google.cloud.datastore_v1.types import query
-
 from .transports.base import DatastoreTransport, DEFAULT_CLIENT_INFO
 from .transports.grpc import DatastoreGrpcTransport
 from .transports.grpc_asyncio import DatastoreGrpcAsyncIOTransport
@@ -54,7 +53,7 @@ class DatastoreClientMeta(type):
     _transport_registry["grpc_asyncio"] = DatastoreGrpcAsyncIOTransport
 
     def get_transport_class(cls, label: str = None,) -> Type[DatastoreTransport]:
-        """Return an appropriate transport class.
+        """Returns an appropriate transport class.
 
         Args:
             label: The name of the desired transport. If none is
@@ -84,7 +83,8 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
     @staticmethod
     def _get_default_mtls_endpoint(api_endpoint):
-        """Convert api endpoint to mTLS endpoint.
+        """Converts api endpoint to mTLS endpoint.
+
         Convert "*.sandbox.googleapis.com" and "*.googleapis.com" to
         "*.mtls.sandbox.googleapis.com" and "*.mtls.googleapis.com" respectively.
         Args:
@@ -116,10 +116,27 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         DEFAULT_ENDPOINT
     )
 
+    @classmethod
+    def from_service_account_info(cls, info: dict, *args, **kwargs):
+        """Creates an instance of this client using the provided credentials
+            info.
+
+        Args:
+            info (dict): The service account private key info.
+            args: Additional arguments to pass to the constructor.
+            kwargs: Additional arguments to pass to the constructor.
+
+        Returns:
+            DatastoreClient: The constructed client.
+        """
+        credentials = service_account.Credentials.from_service_account_info(info)
+        kwargs["credentials"] = credentials
+        return cls(*args, **kwargs)
+
     @classmethod
     def from_service_account_file(cls, filename: str, *args, **kwargs):
         """Creates an instance of this client using the provided credentials
-        file.
+            file.
 
         Args:
             filename (str): The path to the service account private key json
@@ -128,7 +145,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
             kwargs: Additional arguments to pass to the constructor.
 
         Returns:
-            {@api.name}: The constructed client.
+            DatastoreClient: The constructed client.
         """
         credentials = service_account.Credentials.from_service_account_file(filename)
         kwargs["credentials"] = credentials
@@ -138,16 +155,17 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
     @property
     def transport(self) -> DatastoreTransport:
-        """Return the transport used by the client instance.
+        """Returns the transport used by the client instance.
 
         Returns:
-            DatastoreTransport: The transport used by the client instance.
+            DatastoreTransport: The transport used by the client
+                instance.
         """
         return self._transport
 
     @staticmethod
     def common_billing_account_path(billing_account: str,) -> str:
-        """Return a fully-qualified billing_account string."""
+        """Returns a fully-qualified billing_account string."""
         return "billingAccounts/{billing_account}".format(
             billing_account=billing_account,
         )
@@ -160,7 +178,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
     @staticmethod
     def common_folder_path(folder: str,) -> str:
-        """Return a fully-qualified folder string."""
+        """Returns a fully-qualified folder string."""
         return "folders/{folder}".format(folder=folder,)
 
     @staticmethod
@@ -171,7 +189,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
     @staticmethod
     def common_organization_path(organization: str,) -> str:
-        """Return a fully-qualified organization string."""
+        """Returns a fully-qualified organization string."""
         return "organizations/{organization}".format(organization=organization,)
 
     @staticmethod
@@ -182,7 +200,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
     @staticmethod
     def common_project_path(project: str,) -> str:
-        """Return a fully-qualified project string."""
+        """Returns a fully-qualified project string."""
         return "projects/{project}".format(project=project,)
 
     @staticmethod
@@ -193,7 +211,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
     @staticmethod
     def common_location_path(project: str, location: str,) -> str:
-        """Return a fully-qualified location string."""
+        """Returns a fully-qualified location string."""
         return "projects/{project}/locations/{location}".format(
             project=project, location=location,
         )
@@ -207,12 +225,12 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
     def __init__(
         self,
         *,
-        credentials: Optional[credentials.Credentials] = None,
+        credentials: Optional[ga_credentials.Credentials] = None,
         transport: Union[str, DatastoreTransport, None] = None,
         client_options: Optional[client_options_lib.ClientOptions] = None,
         client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
     ) -> None:
-        """Instantiate the datastore client.
+        """Instantiates the datastore client.
 
         Args:
             credentials (Optional[google.auth.credentials.Credentials]): The
@@ -220,10 +238,10 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
                 credentials identify the application to the service; if none
                 are specified, the client will attempt to ascertain the
                 credentials from the environment.
-            transport (Union[str, ~.DatastoreTransport]): The
+            transport (Union[str, DatastoreTransport]): The
                 transport to use. If set to None, a transport is chosen
                 automatically.
-            client_options (client_options_lib.ClientOptions): Custom options for the
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                 client. It won't take effect if a ``transport`` instance is provided.
                 (1) The ``api_endpoint`` property can be used to override the
                 default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
@@ -259,21 +277,18 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
             util.strtobool(os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"))
         )
 
-        ssl_credentials = None
+        client_cert_source_func = None
         is_mtls = False
         if use_client_cert:
             if client_options.client_cert_source:
-                import grpc  # type: ignore
-
-                cert, key = client_options.client_cert_source()
-                ssl_credentials = grpc.ssl_channel_credentials(
-                    certificate_chain=cert, private_key=key
-                )
                 is_mtls = True
+                client_cert_source_func = client_options.client_cert_source
             else:
-                creds = SslCredentials()
-                is_mtls = creds.is_mtls
-                ssl_credentials = creds.ssl_credentials if is_mtls else None
+                is_mtls = mtls.has_default_client_cert_source()
+                if is_mtls:
+                    client_cert_source_func = mtls.default_client_cert_source()
+                else:
+                    client_cert_source_func = None
 
         # Figure out which api endpoint to use.
         if client_options.api_endpoint is not None:
@@ -285,12 +300,14 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
             elif use_mtls_env == "always":
                 api_endpoint = self.DEFAULT_MTLS_ENDPOINT
             elif use_mtls_env == "auto":
-                api_endpoint = (
-                    self.DEFAULT_MTLS_ENDPOINT if is_mtls else self.DEFAULT_ENDPOINT
-                )
+                if is_mtls:
+                    api_endpoint = self.DEFAULT_MTLS_ENDPOINT
+                else:
+                    api_endpoint = self.DEFAULT_ENDPOINT
             else:
                 raise MutualTLSChannelError(
-                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted values: never, auto, always"
+                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
+                    "values: never, auto, always"
                 )
 
         # Save or instantiate the transport.
@@ -305,8 +322,8 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
                 )
             if client_options.scopes:
                 raise ValueError(
-                    "When providing a transport instance, "
-                    "provide its scopes directly."
+                    "When providing a transport instance, provide its scopes "
+                    "directly."
                 )
             self._transport = transport
         else:
@@ -316,46 +333,48 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
                 credentials_file=client_options.credentials_file,
                 host=api_endpoint,
                 scopes=client_options.scopes,
-                ssl_channel_credentials=ssl_credentials,
+                client_cert_source_for_mtls=client_cert_source_func,
                 quota_project_id=client_options.quota_project_id,
                 client_info=client_info,
+                always_use_jwt_access=True,
             )
 
     def lookup(
         self,
-        request: datastore.LookupRequest = None,
+        request: Union[datastore.LookupRequest, dict] = None,
         *,
         project_id: str = None,
         read_options: datastore.ReadOptions = None,
         keys: Sequence[entity.Key] = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.LookupResponse:
         r"""Looks up entities by key.
 
         Args:
-            request (:class:`~.datastore.LookupRequest`):
+            request (Union[google.cloud.datastore_v1.types.LookupRequest, dict]):
                 The request object. The request for
                 [Datastore.Lookup][google.datastore.v1.Datastore.Lookup].
-            project_id (:class:`str`):
+            project_id (str):
                 Required. The ID of the project
                 against which to make the request.
+
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            read_options (:class:`~.datastore.ReadOptions`):
+            read_options (google.cloud.datastore_v1.types.ReadOptions):
                 The options for this lookup request.
                 This corresponds to the ``read_options`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            keys (:class:`Sequence[~.entity.Key]`):
+            keys (Sequence[google.cloud.datastore_v1.types.Key]):
                 Required. Keys of entities to look
                 up.
+
                 This corresponds to the ``keys`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -363,7 +382,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.LookupResponse:
+            google.cloud.datastore_v1.types.LookupResponse:
                 The response for
                 [Datastore.Lookup][google.datastore.v1.Datastore.Lookup].
 
@@ -384,17 +403,14 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         # there are no flattened fields.
         if not isinstance(request, datastore.LookupRequest):
             request = datastore.LookupRequest(request)
-
             # If we have keyword arguments corresponding to fields on the
             # request, apply these.
-
             if project_id is not None:
                 request.project_id = project_id
             if read_options is not None:
                 request.read_options = read_options
-
-            if keys:
-                request.keys.extend(keys)
+            if keys is not None:
+                request.keys = keys
 
         # Wrap the RPC method; this adds retry and timeout information,
         # and friendly error handling.
@@ -408,19 +424,18 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
     def run_query(
         self,
-        request: datastore.RunQueryRequest = None,
+        request: Union[datastore.RunQueryRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.RunQueryResponse:
         r"""Queries for entities.
 
         Args:
-            request (:class:`~.datastore.RunQueryRequest`):
+            request (Union[google.cloud.datastore_v1.types.RunQueryRequest, dict]):
                 The request object. The request for
                 [Datastore.RunQuery][google.datastore.v1.Datastore.RunQuery].
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -428,13 +443,12 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.RunQueryResponse:
+            google.cloud.datastore_v1.types.RunQueryResponse:
                 The response for
                 [Datastore.RunQuery][google.datastore.v1.Datastore.RunQuery].
 
         """
         # Create or coerce a protobuf request object.
-
         # Minor optimization to avoid making a copy if the user passes
         # in a datastore.RunQueryRequest.
         # There's no risk of modifying the input as we've already verified
@@ -454,26 +468,26 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
     def begin_transaction(
         self,
-        request: datastore.BeginTransactionRequest = None,
+        request: Union[datastore.BeginTransactionRequest, dict] = None,
         *,
         project_id: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.BeginTransactionResponse:
         r"""Begins a new transaction.
 
         Args:
-            request (:class:`~.datastore.BeginTransactionRequest`):
+            request (Union[google.cloud.datastore_v1.types.BeginTransactionRequest, dict]):
                 The request object. The request for
                 [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
-            project_id (:class:`str`):
+            project_id (str):
                 Required. The ID of the project
                 against which to make the request.
+
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -481,7 +495,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.BeginTransactionResponse:
+            google.cloud.datastore_v1.types.BeginTransactionResponse:
                 The response for
                 [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
 
@@ -502,10 +516,8 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         # there are no flattened fields.
         if not isinstance(request, datastore.BeginTransactionRequest):
             request = datastore.BeginTransactionRequest(request)
-
             # If we have keyword arguments corresponding to fields on the
             # request, apply these.
-
             if project_id is not None:
                 request.project_id = project_id
 
@@ -521,13 +533,13 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
     def commit(
         self,
-        request: datastore.CommitRequest = None,
+        request: Union[datastore.CommitRequest, dict] = None,
         *,
         project_id: str = None,
         mode: datastore.CommitRequest.Mode = None,
         transaction: bytes = None,
         mutations: Sequence[datastore.Mutation] = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.CommitResponse:
@@ -535,30 +547,33 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         or modifying some entities.
 
         Args:
-            request (:class:`~.datastore.CommitRequest`):
+            request (Union[google.cloud.datastore_v1.types.CommitRequest, dict]):
                 The request object. The request for
                 [Datastore.Commit][google.datastore.v1.Datastore.Commit].
-            project_id (:class:`str`):
+            project_id (str):
                 Required. The ID of the project
                 against which to make the request.
+
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            mode (:class:`~.datastore.CommitRequest.Mode`):
+            mode (google.cloud.datastore_v1.types.CommitRequest.Mode):
                 The type of commit to perform. Defaults to
                 ``TRANSACTIONAL``.
+
                 This corresponds to the ``mode`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            transaction (:class:`bytes`):
+            transaction (bytes):
                 The identifier of the transaction associated with the
                 commit. A transaction identifier is returned by a call
                 to
                 [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
+
                 This corresponds to the ``transaction`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            mutations (:class:`Sequence[~.datastore.Mutation]`):
+            mutations (Sequence[google.cloud.datastore_v1.types.Mutation]):
                 The mutations to perform.
 
                 When mode is ``TRANSACTIONAL``, mutations affecting a
@@ -573,10 +588,10 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
                 When mode is ``NON_TRANSACTIONAL``, no two mutations may
                 affect a single entity.
+
                 This corresponds to the ``mutations`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -584,7 +599,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.CommitResponse:
+            google.cloud.datastore_v1.types.CommitResponse:
                 The response for
                 [Datastore.Commit][google.datastore.v1.Datastore.Commit].
 
@@ -605,19 +620,16 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         # there are no flattened fields.
         if not isinstance(request, datastore.CommitRequest):
             request = datastore.CommitRequest(request)
-
             # If we have keyword arguments corresponding to fields on the
             # request, apply these.
-
             if project_id is not None:
                 request.project_id = project_id
             if mode is not None:
                 request.mode = mode
             if transaction is not None:
                 request.transaction = transaction
-
-            if mutations:
-                request.mutations.extend(mutations)
+            if mutations is not None:
+                request.mutations = mutations
 
         # Wrap the RPC method; this adds retry and timeout information,
         # and friendly error handling.
@@ -631,34 +643,35 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
     def rollback(
         self,
-        request: datastore.RollbackRequest = None,
+        request: Union[datastore.RollbackRequest, dict] = None,
         *,
         project_id: str = None,
         transaction: bytes = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.RollbackResponse:
         r"""Rolls back a transaction.
 
         Args:
-            request (:class:`~.datastore.RollbackRequest`):
+            request (Union[google.cloud.datastore_v1.types.RollbackRequest, dict]):
                 The request object. The request for
                 [Datastore.Rollback][google.datastore.v1.Datastore.Rollback].
-            project_id (:class:`str`):
+            project_id (str):
                 Required. The ID of the project
                 against which to make the request.
+
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            transaction (:class:`bytes`):
+            transaction (bytes):
                 Required. The transaction identifier, returned by a call
                 to
                 [Datastore.BeginTransaction][google.datastore.v1.Datastore.BeginTransaction].
+
                 This corresponds to the ``transaction`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -666,10 +679,9 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.RollbackResponse:
-                The response for
-                [Datastore.Rollback][google.datastore.v1.Datastore.Rollback].
-                (an empty message).
+            google.cloud.datastore_v1.types.RollbackResponse:
+                The response for [Datastore.Rollback][google.datastore.v1.Datastore.Rollback].
+                   (an empty message).
 
         """
         # Create or coerce a protobuf request object.
@@ -688,10 +700,8 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         # there are no flattened fields.
         if not isinstance(request, datastore.RollbackRequest):
             request = datastore.RollbackRequest(request)
-
             # If we have keyword arguments corresponding to fields on the
             # request, apply these.
-
             if project_id is not None:
                 request.project_id = project_id
             if transaction is not None:
@@ -709,11 +719,11 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
     def allocate_ids(
         self,
-        request: datastore.AllocateIdsRequest = None,
+        request: Union[datastore.AllocateIdsRequest, dict] = None,
         *,
         project_id: str = None,
         keys: Sequence[entity.Key] = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.AllocateIdsResponse:
@@ -721,24 +731,25 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         referencing an entity before it is inserted.
 
         Args:
-            request (:class:`~.datastore.AllocateIdsRequest`):
+            request (Union[google.cloud.datastore_v1.types.AllocateIdsRequest, dict]):
                 The request object. The request for
                 [Datastore.AllocateIds][google.datastore.v1.Datastore.AllocateIds].
-            project_id (:class:`str`):
+            project_id (str):
                 Required. The ID of the project
                 against which to make the request.
+
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            keys (:class:`Sequence[~.entity.Key]`):
+            keys (Sequence[google.cloud.datastore_v1.types.Key]):
                 Required. A list of keys with
                 incomplete key paths for which to
                 allocate IDs. No key may be
                 reserved/read-only.
+
                 This corresponds to the ``keys`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -746,7 +757,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.AllocateIdsResponse:
+            google.cloud.datastore_v1.types.AllocateIdsResponse:
                 The response for
                 [Datastore.AllocateIds][google.datastore.v1.Datastore.AllocateIds].
 
@@ -767,15 +778,12 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         # there are no flattened fields.
         if not isinstance(request, datastore.AllocateIdsRequest):
             request = datastore.AllocateIdsRequest(request)
-
             # If we have keyword arguments corresponding to fields on the
             # request, apply these.
-
             if project_id is not None:
                 request.project_id = project_id
-
-            if keys:
-                request.keys.extend(keys)
+            if keys is not None:
+                request.keys = keys
 
         # Wrap the RPC method; this adds retry and timeout information,
         # and friendly error handling.
@@ -789,11 +797,11 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
 
     def reserve_ids(
         self,
-        request: datastore.ReserveIdsRequest = None,
+        request: Union[datastore.ReserveIdsRequest, dict] = None,
         *,
         project_id: str = None,
         keys: Sequence[entity.Key] = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.ReserveIdsResponse:
@@ -801,23 +809,24 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         llocated by Cloud Datastore.
 
         Args:
-            request (:class:`~.datastore.ReserveIdsRequest`):
+            request (Union[google.cloud.datastore_v1.types.ReserveIdsRequest, dict]):
                 The request object. The request for
                 [Datastore.ReserveIds][google.datastore.v1.Datastore.ReserveIds].
-            project_id (:class:`str`):
+            project_id (str):
                 Required. The ID of the project
                 against which to make the request.
+
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            keys (:class:`Sequence[~.entity.Key]`):
+            keys (Sequence[google.cloud.datastore_v1.types.Key]):
                 Required. A list of keys with
                 complete key paths whose numeric IDs
                 should not be auto-allocated.
+
                 This corresponds to the ``keys`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -825,7 +834,7 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
                 sent along with the request as metadata.
 
         Returns:
-            ~.datastore.ReserveIdsResponse:
+            google.cloud.datastore_v1.types.ReserveIdsResponse:
                 The response for
                 [Datastore.ReserveIds][google.datastore.v1.Datastore.ReserveIds].
 
@@ -846,15 +855,12 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         # there are no flattened fields.
         if not isinstance(request, datastore.ReserveIdsRequest):
             request = datastore.ReserveIdsRequest(request)
-
             # If we have keyword arguments corresponding to fields on the
             # request, apply these.
-
             if project_id is not None:
                 request.project_id = project_id
-
-            if keys:
-                request.keys.extend(keys)
+            if keys is not None:
+                request.keys = keys
 
         # Wrap the RPC method; this adds retry and timeout information,
         # and friendly error handling.
@@ -866,6 +872,19 @@ class DatastoreClient(metaclass=DatastoreClientMeta):
         # Done; return the response.
         return response
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        """Releases underlying transport's resources.
+
+        .. warning::
+            ONLY use as a context manager if the transport is NOT shared
+            with other clients! Exiting the with block will CLOSE the transport
+            and may cause errors in other clients!
+        """
+        self.transport.close()
+
 
 try:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
